/***************************************************************************
 *   Copyright (C) 2005 by Sebastian Beck                                  *
 *   bastl@server                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "maschinentool.h"

#include <qfile.h>
#include <qdir.h>
#include <qstring.h>
#include <qcstring.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qdatastream.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcolor.h>
#include <qnamespace.h>
#include <qfiledialog.h>
#include <qpushbutton.h>

#include <knuminput.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kiconloader.h>
#include <kicontheme.h>
#include <kurlrequester.h>

#include "maschinendialog.h"

#include <cstdlib>

MaschinenToolView::MaschinenToolView( QWidget *parent ): Maschinendialog( parent )
	{
	QDir::setCurrent( QDir::homeDirPath()+QString("/PCNC") );
	SAVED = TRUE;
	M_COMBO=TRUE;
	maschinendatei = "maschinen.dat";
	steuerdat = "pcnc-plus";
	steuerung->setURL( steuerdat );
	connect( steuerung, SIGNAL( textChanged( const QString & ) ), this,  SLOT( datei_clicked() ) );
	werkzeugdat = "werkzeuge.dat";
	werkzeuge->setURL( werkzeugdat );
	connect( werkzeuge, SIGNAL( textChanged( const QString & ) ), this,  SLOT( datei2_clicked() ) );
	pos = 0;
	QFile f2( maschinendatei );
	if ( f2.open( IO_ReadOnly ) ) 
		{
		QDataStream str2( &f2 );
		int i = 0;
		QCString name;
		while ( ! str2.atEnd()  )
			{
			str2 >> name;
			auswahl->insertItem( QString(name), -1 );
			i++;
			f2.at( i * 128 );
			}
		f2.close();
		}
	if( 0 == getuid() )
		{
		x_rechts = new KProcess;
		*x_rechts << "m1r";
		x_links = new KProcess;
		*x_links << "m1l";
		y_hinten = new KProcess;
		*y_hinten << "m1h";
		y_vorne = new KProcess;
		*y_vorne << "m1v";
		z_oben = new KProcess;
		*z_oben << "m1o";
		z_unten = new KProcess;
		*z_unten << "m1u";
		}
	else
		{
		spiel_x->setMinValue( 0 );
		spiel_y->setMinValue( 0 );
		spiel_z->setMinValue( 0 );
		textLabel3->setText( "Das Spiel kann nur mit Root-Rechten eingestellt werden !" );
		textLabel3->setAlignment( AlignVCenter | AlignHCenter );
		textLabel3->setPaletteForegroundColor( QColor ( "red" ) );
		}
	
	}

MaschinenToolView::~MaschinenToolView()
	{
	}

void MaschinenToolView::closeEvent( QCloseEvent* )
	{
	if ( ! SAVED )
		if ( KMessageBox::Yes == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "Die Maschine war noch nicht abgespeichert.\nSpeichern ?", "Achtung ", KStdGuiItem::yes(), KStdGuiItem::no() ) )
			speichern_clicked();
	
	exit( 0 );
	}

/**
 * Stores the edited data record to data base maschinen.dat
 *A record looks like this:
 *0-35		36		byte	name
 *36		1		byte	play-x
 *37		1		byte	play-y
 *38		1		byte	play-z
 *39-40	2		byte	feed-x
 *41-42	2		byte	feed-y
 *43-44	2		byte	feed-z
 *45-46	2		byte	feed range-x
 *47-48	2		byte	feed range-y
 *49-50	2		byte	feed range-z
 *51		1		byte	options
 *52-53	2		byte	machine factor
 *54-55	2		byte	acceleration
 *56-91	36		byte	controlling programm
 *92-127	36		byte	tools file
 *-----------------------
 *			128	byte
 */
void MaschinenToolView::speichern_clicked()
	{
	QFile f2( maschinendatei );
	if ( f2.open( IO_ReadWrite ) ) 
		{
		QDataStream str2( &f2 );
		if ( pos > 0 )
			{
			f2.at( (pos - 1) * 128 );
			QCString name = maschine->text().utf8();
			name.resize( 32 );
			str2 << name;
			Q_INT8 c;
			c = spiel_x->value();
			str2 << c;
			c = spiel_y->value();
			str2 << c;
			c = spiel_z->value();
			str2 << c;
			Q_INT16 i;
			i = vorschub_x->value();
			str2 << i;
			i = vorschub_y->value();
			str2 << i;
			i = vorschub_z->value();
			str2 << i;
			i = weg_x->value();
			str2 << i;
			i = weg_y->value();
			str2 << i;
			i = weg_z->value();
			str2 << i;
			Q_INT8 optionen = 0;
			if ( kugelroll->isChecked() )
				optionen = optionen | 01;
			str2 << optionen;
			Q_INT16 mfaktor = mf->value();
			str2 << mfaktor;
			Q_INT16 ac = a->value();
			str2 << ac;
			steuerdat = steuerung->url().utf8();
			steuerdat.resize( 32 );
			str2 << steuerdat;
			werkzeugdat = werkzeuge->url().utf8();
			werkzeugdat.resize( 32 );
			str2 << werkzeugdat;
			f2.close();
			SAVED = TRUE;
			}
			
		f2.close();
		}
	else
		{
		KMessageBox::messageBox( 0, KMessageBox::Error, QString("Beim Speichern ist folgender Fehler aufgetreden:\n%1").arg( f2.errorString() ), "Fehler ");
		}
	}

void MaschinenToolView::auswahl_activated()
	{
	M_COMBO = TRUE;
	if ( ! SAVED )
		{
		if ( KMessageBox::Yes == KMessageBox::messageBox ( 0, KMessageBox::QuestionYesNo, "Die Maschine war noch nicht abgespeichert.\nSpeichern ?", "Achtung ", KStdGuiItem::yes(), KStdGuiItem::no() ) )
			{
			speichern_clicked();
			}
		}
	
	SAVED = TRUE;
	pos = auswahl->currentItem();
	maschine->setText( auswahl->currentText() );
	if ( pos > 0 )
		{
		QFile f2( maschinendatei );
		if ( f2.open( IO_ReadOnly ) ) 
			{
			f2.at( (pos - 1) * 128 );
			QDataStream str2( &f2 );
			QCString name;
			str2 >> name >> s_x  >> s_y  >> s_z;
			spiel_x->setValue( s_x );
			spiel_y->setValue( s_y );
			spiel_z->setValue( s_z );
			Q_INT16 w;
			str2 >> w;
			vorschub_x->setValue( w );
			str2 >> w;
			vorschub_y->setValue( w );
			str2 >> w;
			vorschub_z->setValue( w );
			str2 >> w;
			weg_x->setValue( w );
			str2 >> w;
			weg_y->setValue( w );
			str2 >> w;
			weg_z->setValue( w );
			Q_UINT8 optionen;
			str2 >> optionen;
			if ( optionen & 01 )
				kugelroll->setChecked( TRUE );
			else
				kugelroll->setChecked( FALSE );
			str2 >> w;
			mf->setValue( w );
			str2 >> w;
			a->setValue( w );
			str2 >> steuerdat;
			steuerung->setURL( steuerdat );
			str2 >> werkzeugdat;
			werkzeuge->setURL( werkzeugdat );
			}
		}
	else
		{
		maschine->setText( "Neue" );
		s_x = 0;
		s_y = 0;
		s_z = 0;
		spiel_x->setValue( s_x );
		spiel_y->setValue( s_y );
		spiel_z->setValue( s_z );
		vorschub_x->setValue( 0 );
		vorschub_y->setValue( 0 );
		vorschub_z->setValue( 0 );
		weg_x->setValue( 0 );
		weg_y->setValue( 0 );
		weg_z->setValue( 0 );
		kugelroll->setChecked( 0 );
		mf->setValue( 1 );
		a->setValue( 100 );
		steuerdat = "pcnc-plus";
		steuerung->setURL( steuerdat );
		SAVED = FALSE;
		}
	
	M_COMBO = FALSE;
	}

void MaschinenToolView::name_changed()
	{
	if( M_COMBO ) return;
	
	if ( auswahl->currentItem() == 0 )
		{
		auswahl->insertItem ( maschine->text(), -1 );
		auswahl->setCurrentItem ( auswahl->count() - 1 );
		M_COMBO = FALSE;
		}
	else
		{
		auswahl->changeItem ( maschine->text(), auswahl->currentItem() );
		M_COMBO = FALSE;
		}
	
	pos = auswahl->currentItem();
	SAVED = M_COMBO;
	}

void MaschinenToolView::datei_clicked()
	{
	QString s = steuerung->url();
	if( s != "" )
		{
		s.remove( 0, s.findRev( "/" )+1 );
		steuerung->setURL( s );
		SAVED = M_COMBO;
		}
	}

void MaschinenToolView::datei2_clicked()
	{
	QString s = werkzeuge->url();
	if( s != "" )
		{
		s.remove( 0, s.findRev( "/" )+1 );
		werkzeuge->setURL( s );
		SAVED = M_COMBO;
		}
	}

void MaschinenToolView::changed()
	{
	SAVED = M_COMBO;
	}

void MaschinenToolView::x_clicked()
	{
	SAVED = M_COMBO;
	
	if( 0 != getuid() )
		return;
		
	if( s_x != spiel_x->value() )
		{
		SAVED = M_COMBO;
		if( s_x < spiel_x->value() )
			{
			if( ! x_rechts->start() )
				KMessageBox::messageBox( 0, KMessageBox::Error, QString("Steuerprogramm `m1r' nicht ausfhrbar !"), "Fehler ");
			}
		else
			{
			if( spiel_x->value() == -1 )
				spiel_x->setValue( 0 );
			if( ! x_links->start() )
				KMessageBox::messageBox( 0, KMessageBox::Error, QString("Steuerprogramm `m1l' nicht ausfhrbar !"), "Fehler ");
			}
		}
	
	s_x = spiel_x->value();
	}

void MaschinenToolView::y_clicked()
	{
	SAVED = M_COMBO;
	
	if( 0 != getuid() )
		return;
		
	if( s_y != spiel_y->value() )
		{
		if( s_y < spiel_y->value() )
			{
			if( ! y_hinten->start() )
				KMessageBox::messageBox( 0, KMessageBox::Error, QString("Steuerprogramm `m1h' nicht ausfhrbar !"), "Fehler ");
			}
		else
			{
			if( spiel_y->value() == -1 )
				spiel_y->setValue( 0 );
			if( ! y_vorne->start() )
				KMessageBox::messageBox( 0, KMessageBox::Error, QString("Steuerprogramm `m1v' nicht ausfhrbar !"), "Fehler ");
			}
		}
	
	s_y = spiel_y->value();
	}

void MaschinenToolView::z_clicked()
	{
	SAVED = M_COMBO;
	
	if( 0 != getuid() )
		return;
		
	if( s_z != spiel_z->value() )
		{
		if( s_z < spiel_z->value() )
			{
			if( ! z_oben->start() )
				KMessageBox::messageBox( 0, KMessageBox::Error, QString("Steuerprogramm `m1o' nicht ausfhrbar !"), "Fehler ");
			}
		else
			{
			if( spiel_z->value() == -1 )
				spiel_z->setValue( 0 );
			if( ! z_unten->start() )
				KMessageBox::messageBox( 0, KMessageBox::Error, QString("Steuerprogramm `m1u' nicht ausfhrbar !"), "Fehler ");
			}
		}
	
	s_z = spiel_z->value();
	}



#include "maschinentoolview.moc"
